package cn.gygxzc.tina.config.shiro.common

import org.apache.shiro.codec.Base64
import org.apache.shiro.codec.CodecSupport
import org.apache.shiro.codec.Hex
import org.apache.shiro.util.ByteSource
import java.io.File
import java.io.InputStream
import java.io.Serializable
import java.util.*

/**
 * 实现序列化接口,否则 序列化 反序列化 凉凉
 * @author NorthLan
 * @date 2018/4/24
 * @url https://noahlan.com
 */
class ZenSimpleByteSource : ByteSource, Serializable {
    companion object {
        private const val serialVersionUID = 5640224091610182940L
    }

    private var bytes: ByteArray? = null
    private var cachedHex: String? = null
    private var cachedBase64: String? = null

    constructor()
    constructor(bytes: ByteArray?) {
        this.bytes = bytes
    }

    constructor(chars: CharArray?) {
        this.bytes = CodecSupport.toBytes(chars)
    }

    constructor(string: String?) {
        this.bytes = CodecSupport.toBytes(string)
    }

    constructor(source: ByteSource?) {
        this.bytes = source?.bytes
    }

    constructor(file: File?) {
        this.bytes = BytesHelper().getBytes(file)
    }

    constructor(stream: InputStream?) {
        this.bytes = BytesHelper().getBytes(stream)
    }

    override fun toHex(): String? {
        if (this.cachedHex == null) {
            this.cachedHex = Hex.encodeToString(getBytes())
        }
        return this.cachedHex
    }

    override fun isEmpty(): Boolean {
        return this.bytes == null || this.bytes!!.isEmpty()
    }

    override fun getBytes(): ByteArray? {
        return this.bytes
    }

    override fun toBase64(): String {
        if (this.cachedBase64 == null) {
            this.cachedBase64 = Base64.encodeToString(getBytes())
        }
        return this.cachedBase64!!
    }

    override fun toString(): String {
        return toBase64()
    }

    override fun hashCode(): Int {
        return if (isEmpty) {
            0
        } else Arrays.hashCode(this.bytes)
    }

    override fun equals(other: Any?): Boolean {
        if (other === this) {
            return true
        }
        if (other is ByteSource) {
            val bs = other as ByteSource?
            return Arrays.equals(getBytes(), bs!!.bytes)
        }
        return false
    }

    private class BytesHelper : CodecSupport {
        constructor()

        fun getBytes(file: File?): ByteArray {
            return toBytes(file)
        }

        fun getBytes(stream: InputStream?): ByteArray {
            return toBytes(stream)
        }
    }
}